//////////////////////////////////////////////////////////////////////////////////////
//
//						The Bohge Engine License (BEL)
//
//	Copyright (c) 2011-2014 Peng Zhao
//
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in 
//	all copies or substantial portions of the Software. And the logo of 
//	Bohge Engine shall be displayed full screen for more than 3 seconds 
//	when the software is started. Copyright holders are allowed to develop 
//	game edit based on Bohge Engine, The edit must be released under the MIT 
//	open source license if it is going to be published. In no event shall 
//	copyright holders be prohibited from using any code of Bohge Engine 
//	to develop any other analogous game engines.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//
//////////////////////////////////////////////////////////////////////////////////////

//最普通的参数输出最基本的数据
#SHADER_DEFINE SpineTransfrom

	FUNCTION_INPUT = 
	{
    { CAMERA_RESOLUTION_INV,            HIGH, VEC2,   "in_ResolutionInv"},
    { CAMERA_RESOLUTION_HALF_INV,       HIGH, VEC2,   "in_ResolutionHalfInv"},
    { WORLD_POSITION,           HIGH, VEC3,   "in_WorldPosition"},
    { WORLD_SCALE,              HIGH, VEC3,   "in_WorldScale"},
		{ ATTRIBUTE_POSITION, 			HIGH,	VEC2,		"in_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"in_Color" },
	}

	FUNCTION_OUTPUT = 
	{
		{ SCREENSPACE_POSITION, 		HIGH,	VEC4,		"out_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"out_Color" },
	}

#SHADER_CODE
	vec4 out_Position = vec4(in_Position * in_WorldScale.xy *in_ResolutionInv + in_WorldPosition.xy * in_ResolutionHalfInv, 0.0, 1.0);
  vec4 out_Color = in_Color;
#END_CODE
#END_DEFINE

//最普通的参数输出最基本的数据
#SHADER_DEFINE SpineTransfrom3D

	FUNCTION_INPUT = 
	{
		{ ATTRIBUTE_POSITION, 			HIGH,	VEC2,		"in_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"in_Color" },
		{ CAMERA_VIEWPROJ,			HIGH,	MAT4,		"in_ViewPorj" },
		{ LOCALWORLD_TRANSFORM,		HIGH,	MAT4,		"in_Transfrom" },
	}

	FUNCTION_OUTPUT = 
	{
		{ SCREENSPACE_POSITION, 		HIGH,	VEC4,		"out_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"out_Color" },
	}

#SHADER_CODE
	vec4 originPosition = in_Transfrom * vec4(in_Position/1000.0,0.0,1.0);
	vec4 out_Position = in_ViewPorj * originPosition;
	vec4 out_Color = in_Color;
#END_CODE
#END_DEFINE

#SHADER_DEFINE SpineTransfrom2D

	FUNCTION_INPUT =
	{
		{ CAMERA_RESOLUTION_INV,            HIGH, VEC2,   "in_ResolutionInv"},
		{ CAMERA_RESOLUTION_HALF_INV,       HIGH, VEC2,   "in_ResolutionHalfInv"},
		{ WORLD_POSITION,           HIGH, VEC3,   "in_WorldPosition"},
		{ WORLD_SCALE,              HIGH, VEC3,   "in_WorldScale"},
		{ ATTRIBUTE_POSITION, 			HIGH,	VEC2,		"in_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"in_Color" },
	}

	FUNCTION_OUTPUT =
	{
		{ SCREENSPACE_POSITION, 		HIGH,	VEC4,		"out_Position" },
		{ ATTRIBUTE_COLOR0, 			  LOW,	VEC4,		"out_Color" },
	}

#SHADER_CODE
	vec4 out_Position = vec4(in_Position * in_WorldScale.xy *in_ResolutionInv + in_WorldPosition.xy * in_ResolutionHalfInv, 0.0, 1.0);
	vec4 out_Color = in_Color;
#END_CODE
#END_DEFINE


#SHADER_DEFINE SpriteTransfromXY

	FUNCTION_INPUT = 
	{
    { CAMERA_RESOLUTION_INV,            HIGH, VEC2,   "in_ResolutionInv"},
    { CAMERA_RESOLUTION_HALF_INV,       HIGH, VEC2,   "in_ResolutionHalfInv"},
		{ LOCALWORLD_ROTATION,			HIGH,	MAT3,		"in_WorldRotation" },
    { WORLD_POSITION,           HIGH, VEC3,   "in_WorldPosition"},
    { WORLD_SCALE,              HIGH, VEC3,   "in_WorldScale"},
		{ ATTRIBUTE_POSITION, 			HIGH,	VEC4,		"in_Position" },
	}

	FUNCTION_OUTPUT = 
	{
		{ SCREENSPACE_POSITION, 		HIGH,	VEC4,		"out_Position" },
	}

#SHADER_CODE
	vec4 out_Position = vec4((in_WorldRotation * vec3(in_Position.xy * in_WorldScale.xy, 0.0)).xy * in_ResolutionInv + in_WorldPosition.xy * in_ResolutionHalfInv, 0.0, 1.0);
#END_CODE
#END_DEFINE


#SHADER_DEFINE SpineSurface

	FUNCTION_INPUT = 
	{
		{ SURFACE_COLOR,	      LOW,	  VEC4,		"in_TextureColor" },
		{ ATTRIBUTE_COLOR0,	    LOW,	  VEC4,		"in_Color" },
	}

	FUNCTION_OUTPUT = 
	{
		{ SURFACE_COLOR,	LOW,	VEC4,		"out_SurfaceColor" },
	}
	
#SHADER_CODE
    lowp vec4 out_SurfaceColor = in_TextureColor * in_Color;
#END_CODE
#END_DEFINE