

#DEFPARAMS
_MainTex = { "Main Tex", TEXTURE2D, "white" },
_MaskTex = { "Mask Tex", TEXTURE2D, "white" },
_LutTex = { "Lut Tex", TEXTURE2D, "white"},
_Alpha = { "Alpha", FLOAT, "1.0"},
_BlendFirst = {"Blend First", FLOAT, "1.0"},
_ShaderStyle = {"Shader Style", FLOAT, "1.0"},
_MaskPos = {"Mask Pos", VEC2, "300.0,708.0"},
_MaskScale = {"Mask Scale", FLOAT, "1.01"},
#END

#DEFTAG
ShaderName = "ShadingOfLipStick"
RenderQueue = "Transparent+1"
#END

#DEFPASS Always
COLOR_MASK = COLOR_RGBA
ALPAH_MODE = { ALPAH_OFF }
DRAW_MODE = { CULL_FACE_OFF, DEPTH_MASK_OFF, DEPTH_TEST_OFF, DEPTH_FUNCTION_LESS }
STENCIL_MODE = { STENCIL_OFF }
LIGHT_MODE = { ALWAYS }

CGPROGRAM
#pragma vertex vert
#pragma fragment frag
#include "lipstick_3layer.inc"
#include "common.inc"

struct appdata   
{
	float4 vertex : POSITION;
	float2 uv0 : TEXCOORD0;
	float2 uv1 : TEXCOORD1;
};

struct v2f
{
	float2 uv0 : TEXCOORD0; 
	float2 uv1 : TEXCOORD1;  
	float4 vertex : SV_POSITION;
};

sampler2D _MainTex;
sampler2D _MaskTex;
sampler2D _LutTex;
float _Alpha;
float _ShaderStyle;
float _BlendFirst;
float2 _MaskPos;
float _MaskScale;


v2f vert(appdata v)
{
	v2f o;
	o.vertex = UniformNDC(v.vertex);
    float4 facetexpara = float4(1.0/(167.0*_MaskScale),1.0/(122.0*_MaskScale),_MaskPos.x/(-167.0*_MaskScale),_MaskPos.y/(-122.0*_MaskScale));
    o.uv0 = v.uv0*facetexpara.xy+facetexpara.zw;
	o.uv1 = float2(1.0+v.vertex.x,1.0-v.vertex.y)/2.0;
	return o;
}

fixed4 frag(v2f i) : SV_Target
{
	// sample the texture
	fixed4 basecolor = tex2D(_MainTex, i.uv1);
  fixed3 oricolor = basecolor.rgb;
	fixed4 blendcolor = tex2D(_MaskTex, i.uv0);


    fixed4 baselimit = fixed4(clamp(basecolor.rgb,0.0,0.9),basecolor.a);
    fixed4 newcolor = step(1.5, _ShaderStyle)*blendcolor + step(_ShaderStyle, 1.5)*LUT_mini(baselimit, _LutTex);
    basecolor = step(1.5, _ShaderStyle)*basecolor + step(_ShaderStyle, 1.5)*baselimit;

    newcolor.rgb = Blend_First(basecolor.rgb,newcolor.rgb,_BlendFirst);//混合
    basecolor.rgb = newcolor.rgb*blendcolor.a*_Alpha+(1.0-_Alpha*blendcolor.a)*oricolor;//滑动条控制透明度

    return fixed4(basecolor.rgb,1.0);

}
ENDCG
#END
